@@ -17,6 +17,9 @@ from trytond.transaction import Transaction
 from trytond.tools import sortable_values
 from trytond.wizard import Wizard, StateView, StateAction, Button
 from trytond.pool import Pool
+from trytond.modules.company.model import (
+    employee_field, set_employee, reset_employee)
+from trytond.modules.currency.fields import Monetary
 
 from .exceptions import OverpayWarning
 
@@ -59,35 +62,35 @@ class Group(ModelSQL, ModelView):
     __name__ = 'account.payment.group'
     _rec_name = 'number'
     number = fields.Char('Number', required=True, readonly=True)
-    company = fields.Many2One('company.company', 'Company', required=True,
-        readonly=True, select=True, domain=[
-            ('id', If(Eval('context', {}).contains('company'), '=', '!='),
-                Eval('context', {}).get('company', -1)),
-            ])
+    company = fields.Many2One(
+        'company.company', "Company",
+        required=True, readonly=True, select=True)
     journal = fields.Many2One('account.payment.journal', 'Journal',
         required=True, readonly=True, domain=[
             ('company', '=', Eval('company', -1)),
             ],
         depends=['company'])
     kind = fields.Selection(KINDS, 'Kind', required=True, readonly=True)
-    payments = fields.One2Many('account.payment', 'group', 'Payments',
-        readonly=True)
-    currency_digits = fields.Function(fields.Integer("Currency Digits"),
-        'on_change_with_currency_digits')
+    payments = fields.One2Many(
+        'account.payment', 'group', 'Payments', readonly=True,
+        domain=[
+            ('company', '=', Eval('company', -1)),
+            ('journal', '=', Eval('journal', -1)),
+            ],
+        depends=['company', 'journal'])
+    currency = fields.Function(fields.Many2One(
+            'currency.currency', "Currency"),
+        'on_change_with_currency')
     payment_count = fields.Function(fields.Integer(
             "Payment Count",
             help="The number of payments in the group."),
         'get_payment_aggregated')
-    payment_amount = fields.Function(fields.Numeric(
-            "Payment Total Amount",
-            digits=(16, Eval('currency_digits', 2)),
-            depends=['currency_digits'],
+    payment_amount = fields.Function(Monetary(
+            "Payment Total Amount", currency='currency', digits='currency',
             help="The sum of all payment amounts."),
         'get_payment_aggregated')
-    payment_amount_succeeded = fields.Function(fields.Numeric(
-            "Payment Succeeded",
-            digits=(16, Eval('currency_digits', 2)),
-            depends=['currency_digits'],
+    payment_amount_succeeded = fields.Function(Monetary(
+            "Payment Succeeded", currency='currency', digits='currency',
             help="The sum of the amounts of the successful payments."),
         'get_payment_aggregated')
     payment_complete = fields.Function(fields.Boolean(
@@ -95,6 +98,20 @@ class Group(ModelSQL, ModelView):
             help="All the payments in the group are complete."),
         'get_payment_aggregated', searcher='search_complete')
 
+    process_method = fields.Function(
+        fields.Selection('get_process_methods', "Process Method"),
+        'on_change_with_process_method', searcher='search_process_method')
+
+    @classmethod
+    def __setup__(cls):
+        super().__setup__()
+        cls._buttons.update(
+            succeed={
+                'invisible': Eval('payment_complete', False),
+                'depends': ['payment_complete', 'process_method'],
+                },
+            )
+
     @classmethod
     def __register__(cls, module_name):
         table_h = cls.__table_handler__(module_name)
@@ -108,19 +125,35 @@ class Group(ModelSQL, ModelView):
     def default_company():
         return Transaction().context.get('company')
 
+    @classmethod
+    def get_process_methods(cls):
+        pool = Pool()
+        Journal = pool.get('account.payment.journal')
+        field_name = 'process_method'
+        return Journal.fields_get([field_name])[field_name]['selection']
+
+    @fields.depends('journal')
+    def on_change_with_process_method(self, name=None):
+        if self.journal:
+            return self.journal.process_method
+
+    @classmethod
+    def search_process_method(cls, name, clause):
+        return [('journal.' + clause[0],) + tuple(clause[1:])]
+
     @classmethod
     def create(cls, vlist):
         pool = Pool()
-        Sequence = pool.get('ir.sequence')
         Config = pool.get('account.configuration')
 
         vlist = [v.copy() for v in vlist]
         config = Config(1)
+        default_company = cls.default_company()
         for values in vlist:
             if values.get('number') is None:
-                values['number'] = Sequence.get_id(
-                    config.payment_group_sequence.id)
-
+                values['number'] = config.get_multivalue(
+                    'payment_group_sequence',
+                    company=values.get('company', default_company)).get()
         return super(Group, cls).create(vlist)
 
     @classmethod
@@ -129,9 +162,18 @@ class Group(ModelSQL, ModelView):
             default = {}
         else:
             default = default.copy()
+        default.setdefault('number', None)
         default.setdefault('payments', None)
         return super(Group, cls).copy(groups, default=default)
 
+    @classmethod
+    @ModelView.button
+    def succeed(cls, groups):
+        pool = Pool()
+        Payment = pool.get('account.payment')
+        payments = sum((g.payments for g in groups), ())
+        Payment.succeed(payments)
+
     @classmethod
     def _get_complete_states(cls):
         return ['succeeded', 'failed']
@@ -145,7 +187,7 @@ class Group(ModelSQL, ModelView):
         payment = Payment.__table__()
 
         # initialize result and columns
-        result = defaultdict(defaultdict)
+        result = defaultdict(lambda: defaultdict(lambda: None))
         columns = [
             payment.group.as_('group_id'),
             Count(payment.group).as_('payment_count'),
@@ -222,9 +264,9 @@ class Group(ModelSQL, ModelView):
             return []
 
     @fields.depends('journal')
-    def on_change_with_currency_digits(self, name=None):
+    def on_change_with_currency(self, name=None):
         if self.journal:
-            return self.journal.currency.digits
+            return self.journal.currency.id
 
 
 _STATES = {
@@ -236,12 +278,9 @@ _DEPENDS = ['state']
 class Payment(Workflow, ModelSQL, ModelView):
     'Payment'
     __name__ = 'account.payment'
-    company = fields.Many2One('company.company', 'Company', required=True,
-        select=True, states=_STATES, domain=[
-            ('id', If(Eval('context', {}).contains('company'), '=', '!='),
-                Eval('context', {}).get('company', -1)),
-            ],
-        depends=_DEPENDS)
+    company = fields.Many2One(
+        'company.company', "Company", required=True, select=True,
+        states=_STATES, depends=_DEPENDS)
     journal = fields.Many2One('account.payment.journal', 'Journal',
         required=True, states=_STATES, domain=[
             ('company', '=', Eval('company', -1)),
@@ -250,16 +289,18 @@ class Payment(Workflow, ModelSQL, ModelView):
     currency = fields.Function(fields.Many2One('currency.currency',
             'Currency'), 'on_change_with_currency',
         searcher='search_currency')
-    currency_digits = fields.Function(fields.Integer('Currency Digits'),
-        'on_change_with_currency_digits')
     kind = fields.Selection(KINDS, 'Kind', required=True,
         states=_STATES, depends=_DEPENDS)
-    party = fields.Many2One('party.party', 'Party', required=True,
-        states=_STATES, depends=_DEPENDS)
+    party = fields.Many2One(
+        'party.party', "Party", required=True, states=_STATES,
+        context={
+            'company': Eval('company', -1),
+            },
+        depends=_DEPENDS + ['company'])
     date = fields.Date('Date', required=True, states=_STATES, depends=_DEPENDS)
-    amount = fields.Numeric('Amount', required=True,
-        digits=(16, Eval('currency_digits', 2)), states=_STATES,
-        depends=_DEPENDS + ['currency_digits'])
+    amount = Monetary(
+        "Amount", currency='currency', digits='currency', required=True,
+        states=_STATES, depends=_DEPENDS)
     line = fields.Many2One('account.move.line', 'Line', ondelete='RESTRICT',
         domain=[
             ('move.company', '=', Eval('company', -1)),
@@ -307,6 +348,14 @@ class Payment(Workflow, ModelSQL, ModelView):
     process_method = fields.Function(
         fields.Selection('get_process_methods', "Process Method"),
         'on_change_with_process_method', searcher='search_process_method')
+    approved_by = employee_field(
+        "Approved by",
+        states=['approved', 'processing', 'succeeded', 'failed'])
+    succeeded_by = employee_field(
+        "Success Noted by", states=['succeeded', 'processing'])
+    failed_by = employee_field(
+        "Failure Noted by",
+        states=['failed', 'processing'])
     state = fields.Selection([
             ('draft', 'Draft'),
             ('approved', 'Approved'),
@@ -382,12 +431,6 @@ class Payment(Workflow, ModelSQL, ModelView):
         if self.journal:
             return self.journal.currency.id
 
-    @fields.depends('journal')
-    def on_change_with_currency_digits(self, name=None):
-        if self.journal:
-            return self.journal.currency.digits
-        return 2
-
     @classmethod
     def search_currency(cls, name, clause):
         return [('journal.' + clause[0],) + tuple(clause[1:])]
@@ -415,11 +458,9 @@ class Payment(Workflow, ModelSQL, ModelView):
     @classmethod
     def get_origin(cls):
         Model = Pool().get('ir.model')
+        get_name = Model.get_name
         models = cls._get_origin()
-        models = Model.search([
-                ('model', 'in', models),
-                ])
-        return [(None, '')] + [(m.model, m.name) for m in models]
+        return [(None, '')] + [(m, get_name(m)) for m in models]
 
     @fields.depends('journal')
     def on_change_with_process_method(self, name=None):
@@ -444,6 +485,9 @@ class Payment(Workflow, ModelSQL, ModelView):
         else:
             default = default.copy()
         default.setdefault('group', None)
+        default.setdefault('approved_by')
+        default.setdefault('succeeded_by')
+        default.setdefault('failed_by')
         return super().copy(payments, default=default)
 
     @classmethod
@@ -458,12 +502,14 @@ class Payment(Workflow, ModelSQL, ModelView):
     @classmethod
     @ModelView.button
     @Workflow.transition('draft')
+    @reset_employee('approved_by', 'succeeded_by', 'failed_by')
     def draft(cls, payments):
         pass
 
     @classmethod
     @ModelView.button
     @Workflow.transition('approved')
+    @set_employee('approved_by')
     def approve(cls, payments):
         pass
 
@@ -495,12 +541,14 @@ class Payment(Workflow, ModelSQL, ModelView):
     @classmethod
     @ModelView.button
     @Workflow.transition('succeeded')
+    @set_employee('succeeded_by')
     def succeed(cls, payments):
         pass
 
     @classmethod
     @ModelView.button
     @Workflow.transition('failed')
+    @set_employee('failed_by')
     def fail(cls, payments):
         pass
 
